@@ -4,21 +4,28 @@ from scipy.stats import norm
 
 class GeometricBrownianMotion:
 
-    def simulate_path(self, S, mu, sigma, dt, T):
-        prev_price = S
-        prices = []
-        step = 0
-        while step < T:
-            ds = prev_price*mu*dt + prev_price*sigma*np.random.randn()*np.sqrt(dt)
-            prev_price = prev_price+ds
-            prices.append((prev_price))
-            step += dt
-        return prices
-
-    def __init__(self, S, mu, sigma, dt, T):
-        self.simulated_path = self.simulate_path(S, mu, sigma, dt, T)
+    def vectorized_paths(self, S0, mu, sigma, steps, T, paths):
+        # Discretized change in time
+        dt = T/steps
+        # Matrix of Brownian motions, and ABM/GBM price processes
+        Z = np.random.normal(0, 1, (paths, steps))
+        S = np.zeros((paths, steps+1))
+        X = np.zeros((paths, steps+1))
+        # Compute log transformation
+        X[:, 0] = np.log(S0)
+        for i in range(0, steps):
+            # Standardize cross-sectionally in Z to aid in theoretical convergence
+            if paths > 1:
+                Z[:, i] = ((Z[:, i] - np.mean(Z[:, i]))/np.std(Z[:, i]))
+            X[:, i+1] = X[:, i] + (mu - .5*sigma*sigma)*dt + sigma*Z[:, i]*np.power(dt, .5)
+        S = np.exp(X)  # exp of ABM
+        return S
 
+    def __init__(self, S, mu, sigma, steps, T, paths=1):
+        self.simulated_paths =  self.vectorized_paths(S, mu, sigma, steps, T, paths)
+        self.simulated_path = self.simulated_paths[-1]
 
+# TODO: Vectorize
 class StochasticVarianceModel:
 
     def simulate_path(self, S, mu, r, div, alpha, beta, rho, vol_var, inst_var, dt, T):
@@ -30,7 +37,6 @@ class StochasticVarianceModel:
         while step < T:
             e1 = norm.ppf(np.random.random())
             e2 = e1*rho + np.sqrt(1-(rho**2))*norm.ppf(np.random.random())
-
             price_now = price_now + (r - div) * price_now * dt + price_now * np.sqrt(prev_inst_var * dt) * e1
             prev_inst_var = inst_var_now
             inst_var_now = prev_inst_var + alpha*(beta - prev_inst_var)*dt + vol_var*np.sqrt(prev_inst_var*dt)*e2
@@ -41,12 +47,14 @@ class StochasticVarianceModel:
             else:
                 inst_var_now = .0000001
             prices.append(price_now)
-            step += dt
+            if step + dt > T:
+                dt = T-step
+            else:
+                step += dt
         return prices
 
     def __init__(self, S, mu, r, div, alpha, beta, rho, vol_var, inst_var, dt, T):
-        self.simulated_path = self.simulate_path(S, mu, r, div, alpha, beta, rho, vol_var, inst_var, dt, T)
-
+        self.simulated_paths = self.simulate_path(S, mu, r, div, alpha, beta, rho, vol_var, inst_var, dt, T)
 
 class MonteCarloCall:
 
